'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.apolloServer = undefined;
exports.default = apolloServer;

var _schemaGenerator = require('./schemaGenerator');

var _mock = require('./mock');

var _expressGraphql = require('express-graphql');

var _expressGraphql2 = _interopRequireDefault(_expressGraphql);

var _graphql = require('graphql');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function apolloServer(_ref) // pass through
{
  var schema = _ref.schema;
  var // required
  resolvers = _ref.resolvers;
  var // required if mocks is not false and schema is not GraphQLSchema
  connectors = _ref.connectors;
  var // required if mocks is not false and schema is not GraphQLSchema
  logger = _ref.logger;
  var _ref$mocks = _ref.mocks;
  var mocks = _ref$mocks === undefined ? false : _ref$mocks;
  var _ref$allowUndefinedIn = _ref.allowUndefinedInResolve;
  var allowUndefinedInResolve = _ref$allowUndefinedIn === undefined ? false : _ref$allowUndefinedIn;
  var formatError = _ref.formatError;
  var _ref$graphiql = _ref.graphiql;
  var // pass through
  graphiql = _ref$graphiql === undefined ? true : _ref$graphiql;
  var // pass through
  validationRules = _ref.validationRules;
  var // pass through
  context = _ref.context;
  var // pass through
  rootValue = _ref.rootValue;

  // TODO: throw an error if more than one arg is passed
  // TODO: throw an error if that argument is not an object
  if (!schema) {
    throw new Error('schema is required');
  }
  if (!logger) {
    // eslint-disable-next-line no-param-reassign
    logger = { log: function log(x) {
        return console.log(x);
      } };
  }
  if (!context) {
    // eslint-disable-next-line no-param-reassign
    context = {};
  }
  var executableSchema = void 0;
  if (mocks) {
    var myMocks = mocks || {};
    executableSchema = (0, _mock.mockServer)(schema, myMocks);
  } else {
    // this is just basics, makeExecutableSchema should catch the rest
    // TODO: should be able to provide a GraphQLschema and still use resolvers
    // and connectors if you want, but at the moment this is not possible.
    if (schema instanceof _graphql.GraphQLSchema) {
      executableSchema = (0, _schemaGenerator.makeExecutableSchema)({
        schema: schema,
        resolvers: {},
        connectors: {},
        logger: logger,
        allowUndefinedInResolve: allowUndefinedInResolve
      });
    } else {
      if (!resolvers) {
        throw new Error('resolvers is required option if mocks is not provided');
      }
      if (!connectors) {
        throw new Error('connectors is a required option if mocks is not provided');
      }
      executableSchema = (0, _schemaGenerator.makeExecutableSchema)({
        typeDefs: schema,
        resolvers: resolvers,
        connectors: connectors,
        logger: logger,
        allowUndefinedInResolve: allowUndefinedInResolve
      });
    }
  }

  return function (req, res, next) {
    return (0, _expressGraphql2.default)({
      schema: executableSchema,
      context: context,
      formatError: formatError,
      rootValue: rootValue,
      validationRules: validationRules,
      graphiql: graphiql
    })(req, res, next);
  };
}

exports.apolloServer = apolloServer;